﻿using System;
using System.Collections.Generic;
using System.IO;
using Paladyne.FileSearch.Core;

namespace Paladyne.FileSearch.UI
{
    /// <summary>
    /// Depth-first search implementation of file/directory enumerator (using stack to avoid recursion)
    /// </summary>
    public class DepthFirstFileEnumerator : IFileEnumerator
    {
        #region IFileEnumerator Members

        /// <summary>
        /// Traverse filesystem tree in a bread-first fashion
        /// </summary>
        /// <param name="directoryInfo"></param>
        /// <param name="searchInSubdirectories"></param>
        /// <returns></returns>
        public IEnumerable<FileInfo> Enumerate(DirectoryInfo directoryInfo, bool searchInSubdirectories)
        {
            var stack = new Stack<DirectoryInfo>();
            stack.Push(directoryInfo);

            while (stack.Count > 0)
            {
                DirectoryInfo directory = stack.Pop();

                FileInfo[] files = null;
                try
                {
                    files = directory.GetFiles();
                }
                catch (UnauthorizedAccessException)
                {
                    // Ignore inaccessible directory and its subdirectories
                    continue;
                }
                catch (DirectoryNotFoundException)
                {
                    // Ignore nonexisting directories as well
                    continue;
                }

                foreach (FileInfo file in files)
                {
                    yield return file;
                }

                if (searchInSubdirectories)
                {
                    DirectoryInfo[] directories;
                    try
                    {
                        directories = directory.GetDirectories();
                    }
                    catch (UnauthorizedAccessException)
                    {
                        // Ignore inaccessible directory and its subdirectories
                        continue;
                    }
                    catch (DirectoryNotFoundException)
                    {
                        // Ignore nonexisting directories as well
                        continue;
                    }

                    foreach (DirectoryInfo subDirectory in directories)
                    {
                        stack.Push(subDirectory);
                    }
                }
            }
        }

        #endregion
    }
}